<?php
// vim:tabstop=2:expandtab:shiftwidth=2:filetype=php:syntax:ruler:

/******************************************************************/
/*
/*  LICENSE
/*
/*  This file is part of M.A.R.V.I.N.
/*
/*  M.A.R.V.I.N. is free software; you can redistribute it and/or modify
/*  it under the terms of the GNU General Public License as published by
/*  the Free Software Foundation; either version 2 of the License, or
/*  (at your option) any later version.
/*
/*  M.A.R.V.I.N. is distributed in the hope that it will be useful,
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/*  GNU General Public License for more details.
/*
/*  You should have received a copy of the GNU General Public License
/*  along with M.A.R.V.I.N; if not, write to the Free Software
/*  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 
/*
/*
/*  DESCRIPTION
/*
/*  the PHP class hostInfo parses a given XML file, created by 
/*  hInventory client scripts (Linux and Windows)
/*  it requires PHP to be compiled/installed with xml/expat support
/*  and it is based on the XML output of client scripts taken
/*  from the hInventory project <www.h-inventory.com>
/*
/*
/*  SYNTAX
/*
/*  to use hostInfo, simply drop the following commands in your
/*  PHP file:
/*
/*  include('class_hostInfo.php');
/*  $host =& new hostInfo();
/*  $host->setCfg('./some_host.xml');
/*  $host->parse();
/*
/*  use the following to get a dump of your hostInfo instance
/*  to see what kind of information it fetched from the xml file:
/*
/*  print_r($host);
/* 
/*  if you want some debug information (e.g. components in the xml
/*  file which hostInfo did not recognize) try this:
/*
/*  $host->parse(true);
/*
/*
/*  BACKGROUND
/*  
/*  $this->mode saves the current "data-mode"
/*  the mode is determined by the startHandler/endHandler
/*  functions, which are called by the xml parser for every
/*  <tag> or </tag> found
/*
/*  the function dataHanlder() ist called by the parser for
/*  every character-information which is found between the tags
/*
/*  the "data-mode" tells the dataHandler-function, what kind
/*  of information it is currently working on
/*
/*  e.g. a new component-set in the array $this->output is
/*  created by dataHanlder()
/*  all following attributes (type, name, sub-attributes etc.)
/*  are going to be added to this  component in the
/*  output-array until the parser reaches the end of the component
/*
/*
/*  the DTD of the hInventory xml files:
/* 
/*  <!DOCTYPE computer [
/*  <!ELEMENT computer      (hostname, datetime, component*)>
/*  <!ELEMENT component     (type,name,attr*)>
/*  <!ELEMENT hostname      (#PCDATA)>
/*  <!ELEMENT datetime (#PCDATA)>
/*  <!ELEMENT type          (#PCDATA)>
/*  <!ELEMENT name          (#PCDATA)>
/*  <!ELEMENT attr          (name,value)>
/*  <!ELEMENT value         (#PCDATA)>
/*  ]>
/*
/******************************************************************/

class hostInfo{
   //general vars
   var $xml = null;
   var $fp = false;
	 var $starttime = 0;
	 var $endtime = 0;
	 var $time = 0;

   //parsing related vars
   var $mode = "unknown";
   var $last_attr = "";

   //output related vars
   var $output = array();   

	 var $OS = array("type" => "unknown", "name" => "unknown", "version" => "unknown", "distribution" => "unknown", "release" => "unknown", "servicepack" => "unknown");
	 var $hostname = "unknown";
	 var $datetime = "";
   var $mem = -1;

   var $mainboard_name = "";
   var $mainboard_manuf = "";
   var $mainboard_serial = "";
   var $mainboard_version = "";
   
   //hardware
   var $cpu = array(); //CPUs 
   var $cpucount = 0;
   
   var $disks = array(); //HDDs
   var $diskcount = 0;
   
	 var $fs = array(); //mounted (local) filesystems
   var $fscount = 0;
   
   var $vgs = array(); //LVM Volume Groups (if available)
   var $vgcount = 0;
   
   var $lvs = array(); //LVM Logical Volumes (if available)
   var $lvcount = 0;
   
   var $mds = array(); //Software Raid Devices
   var $mdcount = 0;
   
   var $cddvd = array(); //Optical Drives
   var $cddvdcount = 0;
   
   var $usb = array(); //USB related
   var $usbcount = 0;
   
   var $nics = array(); //Network Interfaces
   var $niccount = 0;
   
   var $gfx = array(); //Grapics Adapter(s)
   var $gfxcount = 0;
   
   var $miscpci = array(); //Misc PCI Hardware
   var $miscpcicount = 0;

   //software
   var $xen = array(); // Xen DomU's
   var $xencount = 0;

   var $vmware2 = array(); // Vmware Server 2 Guests
   var $vmware2count = 0;
   
   var $apps = array(); //installed packages
   var $appcount = 0;
   
   var $updates = array(); //available updates
   var $updatescount = 0;
   
   
   
   function hostInfo(){
    $this->xml = xml_parser_create();
    xml_set_object($this->xml,$this);
    xml_set_character_data_handler($this->xml, 'dataHandler');
    xml_set_element_handler($this->xml, "startHandler", "endHandler");
   }

   function setCfg($file) {
    if(file_exists($file) && is_readable($file)) {
      $this->fp = fopen($file,'r');
    }
    else die("could not open file $file");
   }
   
   function parse($debug = false) {

		$this->starttime = microtime();
		
    if($this->fp) {
      while(($line = fread($this->fp,4096))) {
        xml_parse($this->xml,$line);
      }
      
      if($debug) print_r($this->output);
      
      foreach($this->output AS $element) {
        switch(strtolower($element["type"])) {
          case "operating system":
              $this->OS["name"] = $element["name"];
							$this->OS["version"] = $element["attr"]["version"];
							$this->OS["release"] = $element["attr"]["release"];
							$this->OS["distribution"] = $element["attr"]["distribution"];
              $this->OS["servicepack"] = $element["attr"]["servicepack"];
              if(strpos(strtolower($this->OS["name"]),"linux") !== false) $this->OS["type"] = "linux";
              else if(strpos(strtolower($this->OS["name"]),"windows") !== false) $this->OS["type"] = "windows";
          break;
          case "cpu":
              $count = count($this->cpu);
              $this->cpu[$count]["model"] = $element["name"];
              $this->cpu[$count]["speed"] = $element["attr"]["speed"];
              $this->cpucount = count($this->cpu);
            break;
          case "physical memory":
              $this->mem = $element["attr"]["size"];
            break;
          case "mainboard":
              $this->mainboard_manuf = $element["name"];
              $this->mainboard_version = $element["attr"]["version"];
              $this->mainboard_name = $element["attr"]["product"];
              $this->mainboard_serial = $element["attr"]["serial"];
            break;
          case "hard disk":
              $count = count($this->disks);
              $this->disks[$count]["model"] = $element["name"];
              $this->disks[$count]["size"] = $element["attr"]["size"];
              $this->disks[$count]["interface"] = $element["attr"]["interface"];
              $this->disks[$count]["serial"] = $element["attr"]["serial"];
              $this->diskcount = count($this->disks);
            break;
					case "filesystem":
							$count = count($this->fs);
							$this->fs[$count]["dev"] = $element["name"];
							$this->fs[$count]["fstype"] = $element["attr"]["fstype"];
							$this->fs[$count]["size"] = $element["attr"]["size"];
							$this->fs[$count]["used"] = $element["attr"]["used"];
							$this->fs[$count]["avail"] = $element["attr"]["avail"];
							$this->fs[$count]["use"] = $element["attr"]["use"];
							$this->fs[$count]["mpoint"] = $element["attr"]["mpoint"];
              $this->fscount = count($this->fs);
						break;
          case "volumegroup":
              $count = count($this->vgs);
              $this->vgs[$count]["name"] = $element["name"];
              $this->vgs[$count]["lvcount"] = $element["attr"]["lvnumber"];
              $this->vgs[$count]["size"] = $element["attr"]["size"];
              $this->vgs[$count]["free"] = $element["attr"]["free"];
              $this->vgcount = count($this->vgs);
            break;
          case "logicalvolume":
              $count = count($this->lvs);
              $this->lvs[$count]["name"] = $element["name"];
              $this->lvs[$count]["vg"] = $element["attr"]["vg"];
              $this->lvs[$count]["size"] = $element["attr"]["size"];
              $this->lvcount = count($this->lvs);
            break;
          case "softwareraid":
              $count = count($this->mds);
              $this->mds[$count]["dev"] = $element["name"];
              $this->mds[$count]["level"] = $element["attr"]["level"];
              $this->mds[$count]["sync_state"] = $element["attr"]["state"];
              $this->mds[$count]["devices"] = $element["attr"]["devices"];
              $this->mdcount = count($this->mds);
            break;
          case "cd/dvd drive":
              $count = count($this->cddvd);
              $this->cddvd[$count]["model"] = $element["name"];
              $this->cddvd[$count]["interface"] = $element["attr"]["interface"];
              $this->cddvdcount = count($this->cddvdcount);
            break;
          case "network adapter":
              $count = count($this->nics);
              $this->nics[$count]["model"] = $element["name"];
              $this->nics[$count]["manufacturer"] = $element["attr"]["manufacturer"];
              $this->nics[$count]["mac"] = $element["attr"]["mac"];
              $this->nics[$count]["ip"] = $element["attr"]["ip"];
              $this->nics[$count]["netmask"] = $element["attr"]["subnetmask"];
              $this->niccount = count($this->nics);
            break;
          case "graphic card":
              $count = count($this->gfx);
              $this->gfx[$count]["model"] = $element["name"];
              $this->gfx[$count]["manufacturer"] = $element["attr"]["manufacturer"];
              $this->gfxcount = count($this->gfx);
            break;
					case "bridge":
					case "pci bridge":
					case "isa bridge":
					case "host bridge":
					case "ide controller":
					case "scsi controller":
          case "raid controller":
					case "multimedia audio controller":
  				case "usb controller":
          case "unknown mass storage controller":
          case "mass storage controller":
					case "firewire (ieee 1394)":
							$count = count($this->miscpci);
							$this->miscpci[$count]["type"] = $element["type"];
							$this->miscpci[$count]["model"] = $element["name"];
							$this->miscpci[$count]["manufacturer"] = $element["attr"]["manufacturer"];
              $this->miscpcicount = count($this->miscpci);
						break;
          case "xen":
              $count = count($this->xen);
              $this->xen[$count]["name"] = $element["name"];
              $this->xen[$count]["memory"] = $element["attr"]["memory"];
              $this->xencount = count($this->xen);
            break;
          case "vmware":
              $count = count($this->vmware2);
              $this->vmware2[$count]["name"] = $element["name"];
              $this->vmware2[$count]["state"] = $element["attr"]["state"];
              $this->vmware2count = count($this->vmware2);
            break;
          case "application":
              $count = count($this->apps);
              $this->apps[$count]["name"] = $element["name"];
              $this->appcount = count($this->apps);
            break;
          case "updates":
              $count = count($this->updates);
              $this->updates[$count]["name"] = $element["name"];
              $this->updatescount = count($this->updates);
            break;
          default:
              if($debug) echo "ignored component: ".$element["type"]."\n";
            break;
        } //switch($element...)
      } //foreach...
      
        $this->output = array();
      
    }  //if($this->fp)...

		$this->endtime = microtime();

		$this->time = $this->endtime - $this->starttime;
		
   } //function parse()
   
   function startHandler($parser, $name, $attribs){
        
    $last_idx = count($this->output) - 1;
    
    switch($name) {
			case "HOSTNAME":
					if($this->mode != "attr") $this->mode = "hostname";
				break;
		  case "DATETIME":
					if($this->mode != "attr") $this->mode = "datetime";
				break;
      case "COMPONENT":
          $this->mode = "comp";
          $this->output[] = array(); //new component detected, so add an empty array
        break;
      case "TYPE":
          if($this->mode != "attr") $this->mode = "type";
        break;
      case "NAME":
          if($this->mode != "attr") $this->mode = "name";
          else $this->mode = "attr_name";
        break;
      case "VALUE":
          $this->mode = "attr_value";
        break;
      case "ATTR":
          $this->mode = "attr";
          if(!isset($this->output[$last_idx]["attr"])) {
            $this->output[$last_idx]["attr"] = array();
          }
        break;
    }

   } //function startHandler()

    function dataHandler($parser, $data){
      $last_idx = count($this->output) - 1;

      if(strlen(trim($data)) > 0) {
    
        switch($this->mode) {
          case "attr_name":
              $this->output[$last_idx]["attr"][strtolower($data)] = "";
              $this->last_attr = strtolower($data);
            break;
          case "attr_value":
              if($this->last_attr != "") {
                $this->output[$last_idx]["attr"][$this->last_attr] = $data;
                $this->last_attr = "";
              }
            break;
			    case "hostname":
					    $this->hostname = $data;
				    break;
			    case "datetime":
					    $this->datetime = $data;
				    break;
          case "type":
              $this->output[$last_idx]["type"] = $data;
            break;
          case "name":
            $this->output[$last_idx]["name"] = $data;
            break;
        }
      }
    } //function dataHandler()

   function endHandler($parser, $name){

    switch($name) {
			case "HOSTNAME":
				break;
			case "DATETIME":
					$this->mode = "unknown";
				break;
      case "COMPONENT":
          $this->mode = "unknown";
        break;
      case "TYPE":
          if($this->mode != "attr") $this->mode = "unknown";
        break;
      case "NAME":
          if($this->mode != "attr") $this->mode = "unknown";
          else $this->mode = "unknown";
        break;
      case "VALUE":
          $this->mode = "unknown";
        break;
      case "ATTR":
          $this->mode = "unknown";
        break;
    }
   
   }

}
?>
